import os
from abcli.env import load_env, load_config

load_env(__name__)
load_config(__name__)

# secrets
FIRMS_MAP_KEY = os.getenv(
    "FIRMS_MAP_KEY",
    "",
)

COPERNICUS_AWS_ACCESS_KEY_ID = os.getenv(
    "COPERNICUS_AWS_ACCESS_KEY_ID",
    "",
)

COPERNICUS_AWS_SECRET_ACCESS_KEY = os.getenv(
    "COPERNICUS_AWS_SECRET_ACCESS_KEY",
    "",
)

# QGIS templates
BLUE_GEO_QGIS_TEMPLATE_FIRMS_AREA = os.getenv(
    "BLUE_GEO_QGIS_TEMPLATE_FIRMS_AREA",
    "",
)

BLUE_GEO_QGIS_TEMPLATE_UKRAINE_TIMEMAP = os.getenv(
    "BLUE_GEO_QGIS_TEMPLATE_UKRAINE_TIMEMAP",
    "",
)

BLUE_GEO_QGIS_TEMPLATE_WATCH = os.getenv(
    "BLUE_GEO_QGIS_TEMPLATE_WATCH",
    "",
)


# test datacube-ids
BLUE_GEO_TEST_DATACUBE_COPERNICUS_SENTINEL_2 = os.getenv(
    "BLUE_GEO_TEST_DATACUBE_COPERNICUS_SENTINEL_2",
    "",
)
BLUE_GEO_TEST_DATACUBE_COPERNICUS_SENTINEL_2_CHILCOTIN = os.getenv(
    "BLUE_GEO_TEST_DATACUBE_COPERNICUS_SENTINEL_2_CHILCOTIN",
    "",
)

BLUE_GEO_TEST_DATACUBE_CROP_CTULINE = os.getenv(
    "BLUE_GEO_TEST_DATACUBE_CROP_CTULINE",
    "",
)

BLUE_GEO_TEST_DATACUBE_FIRMS_AREA = os.getenv(
    "BLUE_GEO_TEST_DATACUBE_FIRMS_AREA",
    "",
)

BLUE_GEO_TEST_DATACUBE_GENERIC_GENERIC = os.getenv(
    "BLUE_GEO_TEST_DATACUBE_GENERIC_GENERIC",
    "",
)

BLUE_GEO_TEST_DATACUBE_UKRAINE_TIMEMAP = os.getenv(
    "BLUE_GEO_TEST_DATACUBE_UKRAINE_TIMEMAP",
    "",
)
